@@ -1,4 +1,4 @@
-# Copyright 2022 Observational Health Data Sciences and Informatics
+# Copyright 2023 Observational Health Data Sciences and Informatics
 #
 # This file is part of EvidenceSynthesis
 #
@@ -13,61 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-cleanData <- function(data,
-                      columns,
-                      minValues = rep(-100, length(columns)),
-                      maxValues = rep(100, length(columns)),
-                      grid = FALSE) {
-  for (i in 1:length(columns)) {
-    column <- columns[i]
-    if (any(is.infinite(data[, column]))) {
-      if (grid) {
-        warn(paste("Estimate(s) with infinite log-likelihood detected. Removing before computing meta-analysis."))
-      } else {
-        warn(paste("Estimate(s) with infinite",
-                   column,
-                   "detected. Removing before computing meta-analysis."))
-      }
-      data <- data[!is.infinite(data[, column]), ]
-    }
-    if (any(is.na(data[, column]))) {
-      if (grid) {
-        warn(paste("Estimate(s) with NA log-likelihood detected. Removing before computing meta-analysis."))
-      } else {
-        warn(paste("Estimate(s) with NA",
-                   column,
-                   "detected. Removing before computing meta-analysis."))
-      }
-      data <- data[!is.na(data[, column]), ]
-    }
-    if (any(data[, column] > maxValues[i])) {
-      if (grid) {
-        warn(paste("Estimate(s) with positive log-likelihood detected. Removing before computing meta-analysis."))
-      } else {
-        warn(sprintf("Estimate(s) with extremely high %s (>%s) detected. Removing before computing meta-analysis.",
-                     column,
-                     maxValues[i]))
-      }
-      data <- data[data[, column] <= maxValues[i], ]
-    }
-    if (any(data[, column] < minValues[i])) {
-      if (grid) {
-        warn(paste("Estimate(s) with extremely low log-likelihood detected. Removing before computing meta-analysis."))
-      } else {
-        warn(sprintf("Estimate(s) with extremely low %s (<%s) detected. Removing before computing meta-analysis.",
-                     column,
-                     minValues[i]))
-      }
-      data <- data[data[, column] >= minValues[i], ]
-    }
-  }
-  if (nrow(data) == 0) {
-    warn("No estimates left after removing estimates with NA, infinite or extreme values")
-  }
-  return(data)
-}
-
 createNaEstimate <- function(type) {
   estimate <- data.frame(mu = NA,
                          mu95Lb = NA,
@@ -81,15 +26,6 @@ createNaEstimate <- function(type) {
   return(estimate)
 }
 
-isRmdCheck <- function() {
-  return(Sys.getenv("_R_CHECK_PACKAGE_NAME_", "") != "")
-}
-
-isUnitTest <- function() {
-  return(tolower(Sys.getenv("TESTTHAT", "")) == "true")
-
-}
-
 #' Compute a Bayesian random-effects meta-analysis
 #'
 #' @description
@@ -174,11 +110,9 @@ computeBayesianMetaAnalysis <- function(data,
     Sys.sleep(1) # To avoid CRAN message about CPU time being more than 2.5. times elapsed time
   }
 
-  # Determine type based on data structure:
-  if ("logRr" %in% colnames(data)) {
-    inform("Detected data following normal distribution")
-    type <- "normal"
-    data <- cleanData(data, c("logRr", "seLogRr"), minValues = c(-100, 1e-05))
+  type <- detectApproximationType(data)
+  data <- cleanApproximations(data)
+  if (type == "normal") {
     if (nrow(data) == 0)
       return(createNaEstimate(type))
     dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.NormalDataModel")
@@ -187,10 +121,7 @@ computeBayesianMetaAnalysis <- function(data,
                                         as.numeric(c(NA, NA)))
     }
     dataModel$finish()
-  } else if ("gamma" %in% colnames(data)) {
-    inform("Detected data following custom parameric distribution")
-    type <- "custom"
-    data <- cleanData(data, c("mu", "sigma", "gamma"), minValues = c(-100, 1e-05, -100))
+  } else if (type == "custom") {
     if (nrow(data) == 0)
       return(createNaEstimate(type))
     dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.ParametricDataModel")
@@ -199,13 +130,7 @@ computeBayesianMetaAnalysis <- function(data,
                                         as.numeric(c(NA, NA)))
     }
     dataModel$finish()
-  } else if ("alpha" %in% colnames(data)) {
-    inform("Detected data following skew normal distribution")
-    type <- "skew normal"
-    data <- cleanData(data,
-                      c("mu", "sigma", "alpha"),
-                      minValues = c(-100, 1e-05, -10000),
-                      maxValues = c(100, 10000, 10000))
+  } else if  (type == "skew normal") {
     if (nrow(data) == 0)
       return(createNaEstimate(type))
     dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.SkewNormalDataModel")
@@ -214,62 +139,44 @@ computeBayesianMetaAnalysis <- function(data,
                                         as.numeric(c(NA, NA)))
     }
     dataModel$finish()
-  } else if (is.list(data) && !is.data.frame(data)) {
-    if ("stratumId" %in% names(data[[1]])) {
-      inform("Detected (pooled) patient-level data")
-      type <- "pooled"
-      dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.CoxDataModel")
-      for (i in 1:length(data)) {
-        dataModel$addLikelihoodData(as.integer(data[[i]]$stratumId),
-                                    as.integer(data[[i]]$y),
-                                    as.numeric(data[[i]]$time),
-                                    as.numeric(data[[i]]$x))
-      }
-      dataModel$finish()
-    } else if ("point" %in% names(data[[1]])) {
-      inform("Detected data following adaptive grid distribution")
-      type <- "grid"
-      dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.ExtendingEmpiricalDataModel")
-      for (i in 1:length(data)) {
-        cleanedData <- as.data.frame(data[[i]])
-        cleanedData$value <- cleanedData$value - max(cleanedData$value)
-        cleanedData <- cleanData(cleanedData,
-                                 c("point", "value"),
-                                 minValues = c(-100, -1e6),
-                                 maxValues = c(100, 0))
-        dataModel$addLikelihoodParameters(cleanedData$point, cleanedData$value)
-      }
-      dataModel$finish()
-    } else {
-      abort("Unknown input data format")
+  } else if (type == "pooled") {
+    dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.CoxDataModel")
+    for (i in 1:length(data)) {
+      dataModel$addLikelihoodData(as.integer(data[[i]]$stratumId),
+                                  as.integer(data[[i]]$y),
+                                  as.numeric(data[[i]]$time),
+                                  as.numeric(data[[i]]$x))
     }
-  } else {
-    inform("Detected data following grid distribution")
-    type <- "grid"
+    dataModel$finish()
+  } else if (type == "adaptive grid") {
     dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.ExtendingEmpiricalDataModel")
-    x <- as.numeric(colnames(data))
-    if (any(is.na(x))) {
-      stop("Expecting grid data, but not all column names are numeric")
+    for (i in 1:length(data)) {
+      cleanedData <- as.data.frame(data[[i]])
+      cleanedData$value <- cleanedData$value - max(cleanedData$value)
+      cleanedData <- cleanData(cleanedData,
+                               c("point", "value"),
+                               minValues = c(-100, -1e6),
+                               maxValues = c(100, 0))
+      dataModel$addLikelihoodParameters(cleanedData$point, cleanedData$value)
     }
-    data <- cleanData(data,
-                      colnames(data),
-                      minValues = rep(-1e6, ncol(data)),
-                      maxValues = rep(0, ncol(data)),
-                      grid = TRUE)
+    dataModel$finish()
+  } else if (type == "grid") {
     if (nrow(data) == 0)
       return(createNaEstimate(type))
+    x <- as.numeric(colnames(data))
     data <- as.matrix(data)
+    dataModel <- rJava::.jnew("org.ohdsi.metaAnalysis.ExtendingEmpiricalDataModel")
     for (i in 1:nrow(data)) {
       dataModel$addLikelihoodParameters(x, data[i, ])
     }
     dataModel$finish()
+  } else {
+    abort(sprintf("Approximation type '%s' not supported by this function", type))
   }
 
   inform("Performing MCMC. This may take a while")
-
   prior <- rJava::.jnew("org.ohdsi.metaAnalysis.HalfNormalOnStdDevPrior", 0, as.numeric(priorSd[2]))
-
-  if(robust){
+  if (robust) {
     metaAnalysis <- rJava::.jnew("org.ohdsi.mcmc.Runner",
                                  rJava::.jcast(rJava::.jnew("org.ohdsi.metaAnalysis.RobustMetaAnalysis",
                                                             rJava::.jcast(dataModel, "org.ohdsi.metaAnalysis.DataModel"),
@@ -280,7 +187,7 @@ computeBayesianMetaAnalysis <- function(data,
                                  as.integer(burnIn),
                                  as.integer(subSampleFrequency),
                                  as.numeric(seed))
-  }else{
+  } else {
     metaAnalysis <- rJava::.jnew("org.ohdsi.mcmc.Runner",
                                  rJava::.jcast(rJava::.jnew("org.ohdsi.metaAnalysis.MetaAnalysis",
                                                             rJava::.jcast(dataModel, "org.ohdsi.metaAnalysis.DataModel"),
